/*
 Copyright (C) 2001, 2002, 2003 Sadruddin Rejeb

 This file is part of QuantLib, a free-software/open-source library
 for financial quantitative analysts and developers - http://quantlib.org/

 QuantLib is free software: you can redistribute it and/or modify it under the
 terms of the QuantLib license.  You should have received a copy of the
 license along with this program; if not, please email ferdinando@ametrano.net
 The license is also available online at http://quantlib.org/html/license.html

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the license for more details.
*/
/*! \file swaptionpricer.hpp
    \brief Swaption pricer class

    \fullpath
    ql/Pricers/%swaptionpricer.hpp
*/

// $Id: swaptionpricer.hpp,v 1.6.8.1 2002/12/31 18:40:42 nando Exp $

#ifndef quantlib_pricers_swaption_pricer_h
#define quantlib_pricers_swaption_pricer_h

#include <ql/numericalmethod.hpp>
#include <ql/Instruments/swaption.hpp>
#include <ql/PricingEngines/genericengine.hpp>

namespace QuantLib {

    namespace Pricers {

        class DiscretizedSwap : public DiscretizedAsset {
          public:
            DiscretizedSwap(const Handle<NumericalMethod>& method, 
                            const Instruments::SwaptionArguments& params)
            : DiscretizedAsset(method), arguments_(params) {}

            void reset(Size size) {
                values_ = Array(size, 0.0);
                adjustValues();
            }

            void adjustValues();

            void addTimes(std::list<Time>& times) const {
                Size i;
                for (i=0; i<arguments_.fixedPayTimes.size(); i++)
                    times.push_back(arguments_.fixedPayTimes[i]);
                for (i=0; i<arguments_.floatingResetTimes.size(); i++)
                    times.push_back(arguments_.floatingResetTimes[i]);
                for (i=0; i<arguments_.floatingPayTimes.size(); i++)
                    times.push_back(arguments_.floatingPayTimes[i]);
            }

          private:
            Instruments::SwaptionArguments arguments_;
        };

        class DiscretizedSwaption : public DiscretizedAsset {
          public:
            DiscretizedSwaption(
                const Handle<NumericalMethod>& method,
                const Instruments::SwaptionArguments& params)
            : DiscretizedAsset(method), arguments_(params), 
              swap_(new DiscretizedSwap(method, params)) {
                Time lastFixedPay = arguments_.fixedPayTimes.back();
                Time lastFloatPay = arguments_.floatingPayTimes.back();
                Time start = QL_MAX(lastFixedPay, lastFloatPay);
                method->initialize(swap_, start);
            }

            void reset(Size size) {
                values_ = Array(size, 0.0);
                adjustValues();
            }

            virtual void adjustValues();

            void addTimes(std::list<Time>& times) const {
                swap_->addTimes(times);
                for (Size i=0; i<arguments_.exerciseTimes.size(); i++)
                    times.push_back(arguments_.exerciseTimes[i]);
            }
          private:
            void applySpecificCondition() {
//                std::cout << "Before: " << values_ << std::endl;
                for (Size i=0; i<values_.size(); i++)
                    values_[i] = QL_MAX(swap_->values()[i], values_[i]);
////                std::cout << "After:  " << values_ << std::endl;
            }

            Instruments::SwaptionArguments arguments_;
            Handle<DiscretizedSwap> swap_;
        };

        template<class ModelType>
        class SpecificSwaptionPricer : 
            public PricingEngines::GenericModelEngine<
                    ModelType, 
                    Instruments::SwaptionArguments,
                    Instruments::SwaptionResults > {
          public:
            SpecificSwaptionPricer(const Handle<ModelType>& model) 
            : PricingEngines::GenericModelEngine<
                    ModelType, 
                    Instruments::SwaptionArguments,
                    Instruments::SwaptionResults >(model) {}

            void calculate() const { 
                results_.value = model_->swaption(arguments_); 
            }
        };

    }

}


#endif

